'use strict';

var index_esm = require('./index.esm-CVuJLXJZ.js');
var stateOptions = require('./state-options-CIT9xdTz.js');
var businessFormOptions = require('./business-form-options-CMII7nIK.js');

const transformEmptyString = (value) => {
    return value === '' ? null : value;
};
const validateRoutingNumber = (routingNumber) => {
    if (!routingNumber)
        return false;
    const integers = Array.from(String(routingNumber), Number);
    if (integers.length === 9) {
        const checksum = ((3 * (integers[0] + integers[3] + integers[6])) +
            (7 * (integers[1] + integers[4] + integers[7])) +
            (integers[2] + integers[5] + integers[8])) % 10;
        if (checksum === 0)
            return true;
    }
    return false;
};
const phoneRegex = /^\d{10}$/;
const urlRegex = /^(?:http(s)?:\/\/)?(?:www\.)?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)$/;
const numbersOnlyRegex = /^\d+$/;
const ssnRegex = /^(?!000|666|9\d{2})\d{3}(?!00)\d{2}(?!0000)\d{4}$/;
const poBoxRegex = /P\.?\s?O\.?\s?Box|Postal\sBox/i;

// Common Validations
const emailValidation = index_esm.create$6()
    .email('Enter valid email')
    .transform(transformEmptyString);
const phoneValidation = index_esm.create$6()
    .matches(phoneRegex, 'Enter valid phone number')
    .transform(transformEmptyString);
// Core Info Validations
const businessNameValidation = index_esm.create$6()
    .min(2, 'Name must be at least 2 characters')
    .max(100, 'Name must be less than 100 characters')
    .transform(transformEmptyString);
const doingBusinessAsValidation = index_esm.create$6()
    .min(2, 'Name must be at least 2 characters')
    .max(100, 'Name must be less than 100 characters')
    .transform(transformEmptyString);
const websiteUrlValidation = index_esm.create$6()
    .matches(urlRegex, 'Enter valid website url')
    .transform(transformEmptyString);
const businessClassificationValidation = index_esm.create$6()
    .oneOf(businessFormOptions.businessClassificationOptions.map((option) => option.value), 'Select business classification')
    .transform(transformEmptyString);
const industryValidation = index_esm.create$6()
    .min(2, 'Industry must be at least 2 characters')
    .max(50, 'Industry must be less than 50 characters')
    .transform(transformEmptyString);
const taxIdValidation = index_esm.create$6()
    .matches(numbersOnlyRegex, 'Enter valid tax id, SSN, or EIN')
    .test('not-repeat', 'Enter valid tax id, SSN, or EIN', (value) => {
    return !/^(\d)\1+$/.test(value);
})
    .test('not-seq', 'Enter valid tax id, SSN, or EIN', (value) => {
    return value !== '123456789';
})
    .transform(transformEmptyString);
const dateOfIncorporationValidation = index_esm.create$6()
    .test('not-future', 'Date of incorporation cannot be in the future', (value) => {
    const inputDate = new Date(value);
    const today = new Date();
    return inputDate <= today;
})
    .transform(transformEmptyString);
// Identity Validations
const identityNameValidation = index_esm.create$6()
    .min(2, 'Name must be at least 2 characters')
    .max(100, 'Name must be less than 100 characters')
    .transform(transformEmptyString);
const identityTitleValidation = index_esm.create$6()
    .min(2, 'Title must be at least 2 characters')
    .max(50, 'Title must be less than 50 characters')
    .transform(transformEmptyString);
const dobValidation = (role) => {
    return index_esm.create$6()
        .test('min', 'Enter a valid date', (value) => {
        const date = new Date(value);
        const minDate = new Date('1902-01-01');
        return date >= minDate;
    })
        .test('age', `${role} must be at least 18 years old`, (value) => {
        const date = new Date(value);
        const minAgeDate = new Date();
        minAgeDate.setFullYear(minAgeDate.getFullYear() - 18);
        return date <= minAgeDate;
    })
        .transform(transformEmptyString);
};
const ssnValidation = index_esm.create$6()
    .matches(ssnRegex, 'Enter valid SSN')
    .when('ssn_last4', {
    is: (val) => !val || val.length === 0,
    then: (schema) => schema.required('Enter SSN'),
    otherwise: (schema) => schema.nullable(),
})
    .test('not-repeat', 'Enter valid SSN', (value) => {
    return !/^(\d)\1+$/.test(value);
})
    .test('not-seq', 'Enter valid SSN', (value) => {
    return value !== '123456789';
})
    .transform(transformEmptyString);
// Address Validations
const lineOneValidation = index_esm.create$6()
    .min(5, 'Address must be at least 5 characters')
    .max(100, 'Address must be less than 100 characters')
    .test('not-po-box', 'A PO Box is not a valid address entry', (value) => {
    return !poBoxRegex.test(value);
})
    .transform(transformEmptyString);
const lineTwoValidation = index_esm.create$6()
    .max(100, 'Address must be less than 100 characters')
    .test('not-po-box', 'A PO Box is not a valid address entry', (value) => {
    return !poBoxRegex.test(value);
})
    .transform(transformEmptyString);
const cityValidation = index_esm.create$6()
    .min(2, 'City must be at least 2 characters')
    .max(50, 'City must be less than 50 characters')
    .transform(transformEmptyString);
const stateValidation = index_esm.create$6()
    .oneOf(stateOptions.StateOptions.map((option) => option.value), 'Enter a 2-letter state abbreviation, such as CA')
    .transform(transformEmptyString);
const postalValidation = index_esm.create$6()
    .matches(/^[0-9]{5}$/, 'Enter valid postal code')
    .transform(transformEmptyString);
// Additional Questions Validations
const revenueValidation = index_esm.create$6()
    .matches(numbersOnlyRegex, 'Enter valid revenue')
    .transform(transformEmptyString);
const paymentVolumeValidation = index_esm.create$6()
    .matches(numbersOnlyRegex, 'Enter valid payment volume')
    .transform(transformEmptyString);
const whenServiceReceivedValidation = index_esm.create$6()
    .oneOf(businessFormOptions.businessServiceReceivedOptions.map((option) => option.value), 'Select when service is received')
    .transform(transformEmptyString);
const recurringPaymentsValidation = index_esm.create$6()
    .oneOf(businessFormOptions.recurringPaymentsOptions.map((option) => option.value), 'Select recurring payments')
    .transform(transformEmptyString);
const recurringPaymentsPercentageValidation = index_esm.create$6()
    .when('business_recurring_payments', {
    is: (val) => val === 'Yes',
    then: (schema) => schema.required('Enter recurring payments percentage'),
    otherwise: (schema) => schema.nullable(),
})
    .transform(transformEmptyString);
const seasonalBusinessValidation = index_esm.create$6()
    .oneOf(businessFormOptions.seasonalBusinessOptions.map((option) => option.value), 'Select seasonal business')
    .transform(transformEmptyString);
const otherPaymentDetailsValidation = index_esm.create$6().transform(transformEmptyString);
// Bank Account Validations
const bankNameValidation = index_esm.create$6()
    .min(2, 'Name must be at least 2 characters')
    .max(50, 'Name must be less than 50 characters')
    .transform(transformEmptyString);
const nicknameValidation = index_esm.create$6()
    .min(2, 'Name must be at least 2 characters')
    .max(50, 'Name must be less than 50 characters')
    .transform(transformEmptyString);
const accountTypeValidation = index_esm.create$6()
    .oneOf(businessFormOptions.bankAccountTypeOptions.map((option) => option.value), 'Select account type')
    .transform(transformEmptyString);
const accountNumberValidation = index_esm.create$6()
    .min(4, 'Account number must be at least 4 digits')
    .max(17, 'Account number must be less than 17 digits')
    .matches(numbersOnlyRegex, 'Enter valid account number')
    .test('not-repeat', 'Enter valid account number', (value) => {
    return !/^(\d)\1+$/.test(value);
})
    .transform(transformEmptyString);
const routingNumberValidation = index_esm.create$6()
    .length(9, 'Routing number must be 9 digits')
    .matches(numbersOnlyRegex, 'Enter valid routing number')
    .test('not-repeat', 'Enter valid routing number', (value) => {
    return !/^(\d)\1+$/.test(value);
})
    .test('valid', 'Enter valid routing number', (value) => {
    return validateRoutingNumber(value);
})
    .transform(transformEmptyString);

exports.accountNumberValidation = accountNumberValidation;
exports.accountTypeValidation = accountTypeValidation;
exports.bankNameValidation = bankNameValidation;
exports.businessClassificationValidation = businessClassificationValidation;
exports.businessNameValidation = businessNameValidation;
exports.cityValidation = cityValidation;
exports.dateOfIncorporationValidation = dateOfIncorporationValidation;
exports.dobValidation = dobValidation;
exports.doingBusinessAsValidation = doingBusinessAsValidation;
exports.emailValidation = emailValidation;
exports.identityNameValidation = identityNameValidation;
exports.identityTitleValidation = identityTitleValidation;
exports.industryValidation = industryValidation;
exports.lineOneValidation = lineOneValidation;
exports.lineTwoValidation = lineTwoValidation;
exports.nicknameValidation = nicknameValidation;
exports.otherPaymentDetailsValidation = otherPaymentDetailsValidation;
exports.paymentVolumeValidation = paymentVolumeValidation;
exports.phoneValidation = phoneValidation;
exports.postalValidation = postalValidation;
exports.recurringPaymentsPercentageValidation = recurringPaymentsPercentageValidation;
exports.recurringPaymentsValidation = recurringPaymentsValidation;
exports.revenueValidation = revenueValidation;
exports.routingNumberValidation = routingNumberValidation;
exports.seasonalBusinessValidation = seasonalBusinessValidation;
exports.ssnValidation = ssnValidation;
exports.stateValidation = stateValidation;
exports.taxIdValidation = taxIdValidation;
exports.websiteUrlValidation = websiteUrlValidation;
exports.whenServiceReceivedValidation = whenServiceReceivedValidation;
