import { h } from "@stencil/core";
import { DisputeManagementClickActions } from "../event-types";
import { heading4 } from "../../../styles/parts";
import { Button } from "../../../ui-components/button";
export class DisputeResponseCore {
    constructor() {
        this.disputeResponse = {};
        this.isLoading = false;
        this.documentList = [];
        this.documentErrors = {};
        this.currentStep = 0;
        this.componentStepMapping = [
            () => h("justifi-product-or-service", { ref: (el) => this.currentStepComponentRef = el, disputeResponse: this.disputeResponse, documentErrors: this.documentErrors }),
            () => h("justifi-customer-details", { ref: (el) => this.currentStepComponentRef = el, disputeResponse: this.disputeResponse, documentErrors: this.documentErrors }),
            () => h("justifi-cancellation-policy", { ref: (el) => this.currentStepComponentRef = el, disputeResponse: this.disputeResponse, documentErrors: this.documentErrors }),
            () => h("justifi-refund-policy", { ref: (el) => this.currentStepComponentRef = el, disputeResponse: this.disputeResponse, documentErrors: this.documentErrors }),
            () => h("justifi-duplicate-charge", { ref: (el) => this.currentStepComponentRef = el, disputeResponse: this.disputeResponse, documentErrors: this.documentErrors }),
            () => h("justifi-electronic-evidence", { ref: (el) => this.currentStepComponentRef = el, disputeResponse: this.disputeResponse, documentErrors: this.documentErrors }),
            () => h("justifi-shipping-details", { ref: (el) => this.currentStepComponentRef = el, disputeResponse: this.disputeResponse, documentErrors: this.documentErrors }),
            () => h("justifi-additional-statement", { ref: (el) => this.currentStepComponentRef = el, disputeResponse: this.disputeResponse, documentErrors: this.documentErrors }),
        ];
        this.saveData = async (formData, formStep) => {
            const hasFormData = Object.keys(formData).length;
            if (!hasFormData) {
                this.stepCompleteEvent.emit({ response: null, formStep: formStep });
                return;
            }
            if (this.isLastStep) {
                return this.submitDisputeResponse({
                    payload: formData,
                    onSuccess: (response) => {
                        this.submitEvent.emit({ response: response });
                        this.stepCompleteEvent.emit({ response: response, formStep: formStep });
                    },
                    onError: ({ error, code, severity }) => {
                        this.errorEvent.emit({
                            errorCode: code,
                            message: error,
                            severity,
                        });
                    },
                });
            }
            else {
                return this.updateDisputeResponse({
                    payload: formData,
                    onSuccess: (response) => {
                        this.disputeResponse = Object.assign({}, response.data);
                        this.stepCompleteEvent.emit({ response: response, formStep: formStep });
                    },
                    onError: ({ error, code, severity }) => {
                        this.errorEvent.emit({
                            errorCode: code,
                            message: error,
                            severity,
                        });
                    },
                });
            }
        };
        this.initializeMakePresignedURLs = async () => {
            const presignedUrlPromises = this.documentList.map((document) => {
                return this.getPresignedFileUrl(document);
            });
            return await Promise.all(presignedUrlPromises);
        };
        this.initializeFileUploads = async () => {
            const uploadDocumentPromises = this.documentList.map((document) => {
                return this.uploadDocument(document);
            });
            return Promise.all(uploadDocumentPromises);
        };
        this.getPresignedFileUrl = async (document) => {
            return this.createDisputeEvidence({
                payload: {
                    file_name: document.file_name,
                    file_type: document.file_type,
                    dispute_evidence_type: document.dispute_evidence_type,
                },
                onSuccess: (response) => {
                    document.presignedUrl = response.data.presigned_url;
                },
                onError: ({ error, code, severity }) => {
                    const errors = { [document.dispute_evidence_type]: error };
                    this.documentErrors = Object.assign(Object.assign({}, this.documentErrors), errors);
                    this.errorEvent.emit({
                        errorCode: code,
                        message: error,
                        severity,
                    });
                },
            });
        };
        this.uploadDocument = async (document) => {
            const fileData = await document.getFileString();
            if (!document.presignedUrl) {
                throw new Error('Presigned URL is not set');
            }
            const response = await fetch(document.presignedUrl, {
                method: 'PUT',
                body: fileData,
            });
            return response;
        };
        this.handleSubmit = async (formData, documentList, formStep) => {
            this.isLoading = true;
            if (documentList.length) {
                this.documentErrors = {};
                this.documentList = documentList;
                await this.initializeMakePresignedURLs();
                const hasPresigningErrors = Object.keys(this.documentErrors).length;
                if (hasPresigningErrors) {
                    this.isLoading = false;
                    throw new Error('Could not presign all documents');
                }
                await this.initializeFileUploads();
            }
            // this needs to happen last because it fires the 'submit-event' and 'complete-form-step-event' event
            await this.saveData(formData, formStep);
            this.isLoading = false;
        };
        this.onCancel = () => {
            this.clickEvent.emit({ name: DisputeManagementClickActions.cancelDispute });
        };
        // after each of these steps where validateAndSubmit is called, reload the dispute
        // and set isLoading, and pass defaults into each step
        this.onBack = async () => {
            await this.currentStepComponentRef.validateAndSubmit(async (formData, documentList, formStep) => {
                this.clickEvent.emit({ name: DisputeManagementClickActions.previousStep });
                await this.handleSubmit(formData, documentList, formStep);
                this.currentStep--;
            });
        };
        this.onNext = async () => {
            await this.currentStepComponentRef.validateAndSubmit(async (formData, documentList, formStep) => {
                this.clickEvent.emit({ name: DisputeManagementClickActions.nextStep });
                await this.handleSubmit(formData, documentList, formStep);
                this.currentStep++;
            });
        };
        this.onSubmit = async () => {
            await this.currentStepComponentRef.validateAndSubmit(async (formData, documentList, formStep) => {
                this.clickEvent.emit({ name: DisputeManagementClickActions.submit });
                const submitData = Object.assign(Object.assign({}, formData), { forfeit: false });
                await this.handleSubmit(submitData, documentList, formStep);
            });
        };
    }
    get currentStepComponent() {
        return this.componentStepMapping[this.currentStep]();
    }
    get isLastStep() {
        return this.currentStep === this.componentStepMapping.length - 1;
    }
    get isFirstStep() {
        return this.currentStep === 0;
    }
    render() {
        return (h("div", { key: '09b09843d54a2dbced9788d16e244b2e59a3a78f', class: "row gy-3" }, h("div", { key: '6a3fe32d65ea1a74161d9ba9984543cf950e47ea', class: "col-12" }, h("h1", { key: 'bbb6f1334fd6b120e592117ba191d90b628e620f', class: "h4", part: heading4 }, "Counter dispute")), h("div", { key: '19cbc45842294e9344dbffb725f5f053bf370af7', class: "col-12" }, this.currentStepComponent), h("div", { key: 'fa875ca152ea77fc4c8363216d8ba16c7699b597', class: "col-12" }, h("div", { key: 'fa7b62fff0cbac81c19ebad919f5dd54c3679e84', class: "d-flex gap-2 mt-4 justify-content-end" }, this.isFirstStep ? (h(Button, { variant: "secondary", onClick: () => this.onCancel(), disabled: this.isLoading, isLoading: this.isLoading }, "Cancel")) : (h(Button, { variant: "secondary", onClick: () => this.onBack(), disabled: this.isLoading, isLoading: this.isLoading }, "Back")), this.isLastStep ? (h(Button, { variant: "primary", onClick: () => this.onSubmit(), disabled: this.isLoading, isLoading: this.isLoading }, "Submit Counter Dispute")) : (h(Button, { variant: "primary", onClick: () => this.onNext(), disabled: this.isLoading, isLoading: this.isLoading }, "Next"))))));
    }
    static get is() { return "justifi-dispute-response-core"; }
    static get properties() {
        return {
            "updateDisputeResponse": {
                "type": "unknown",
                "attribute": "update-dispute-response",
                "mutable": false,
                "complexType": {
                    "original": "(args: {\n    payload: any,\n    onSuccess: (disputeResponse: any) => void,\n    onError: (disputeResponse: any) => void\n  }) => Promise<IApiResponse<IDispute>>",
                    "resolved": "(args: { payload: any; onSuccess: (disputeResponse: any) => void; onError: (disputeResponse: any) => void; }) => Promise<IApiResponse<IDispute>>",
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        },
                        "IApiResponse": {
                            "location": "import",
                            "path": "../../../api",
                            "id": "src/api/index.ts::IApiResponse"
                        },
                        "IDispute": {
                            "location": "import",
                            "path": "../../../api/Dispute",
                            "id": "src/api/Dispute.ts::IDispute"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "submitDisputeResponse": {
                "type": "unknown",
                "attribute": "submit-dispute-response",
                "mutable": false,
                "complexType": {
                    "original": "(args: {\n    payload: any,\n    onSuccess: (disputeResponse: any) => void,\n    onError: (disputeResponse: any) => void\n  }) => Promise<IApiResponse<IDispute>>",
                    "resolved": "(args: { payload: any; onSuccess: (disputeResponse: any) => void; onError: (disputeResponse: any) => void; }) => Promise<IApiResponse<IDispute>>",
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        },
                        "IApiResponse": {
                            "location": "import",
                            "path": "../../../api",
                            "id": "src/api/index.ts::IApiResponse"
                        },
                        "IDispute": {
                            "location": "import",
                            "path": "../../../api/Dispute",
                            "id": "src/api/Dispute.ts::IDispute"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "createDisputeEvidence": {
                "type": "unknown",
                "attribute": "create-dispute-evidence",
                "mutable": false,
                "complexType": {
                    "original": "(args: {\n    payload: any,\n    onSuccess: (disputeResponse: any) => void,\n    onError: (disputeResponse: any) => void\n  }) => Promise<IApiResponse<any>>",
                    "resolved": "(args: { payload: any; onSuccess: (disputeResponse: any) => void; onError: (disputeResponse: any) => void; }) => Promise<IApiResponse<any>>",
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        },
                        "IApiResponse": {
                            "location": "import",
                            "path": "../../../api",
                            "id": "src/api/index.ts::IApiResponse"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "disputeResponse": {
                "type": "any",
                "attribute": "dispute-response",
                "mutable": false,
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "{}"
            }
        };
    }
    static get states() {
        return {
            "isLoading": {},
            "documentList": {},
            "documentErrors": {},
            "currentStep": {},
            "currentStepComponentRef": {}
        };
    }
    static get events() {
        return [{
                "method": "clickEvent",
                "name": "click-event",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "ComponentClickEvent",
                    "resolved": "ComponentClickEvent",
                    "references": {
                        "ComponentClickEvent": {
                            "location": "import",
                            "path": "../../../api/ComponentEvents",
                            "id": "src/api/ComponentEvents.ts::ComponentClickEvent"
                        }
                    }
                }
            }, {
                "method": "errorEvent",
                "name": "error-event",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "ComponentErrorEvent",
                    "resolved": "ComponentErrorEvent",
                    "references": {
                        "ComponentErrorEvent": {
                            "location": "import",
                            "path": "../../../api/ComponentEvents",
                            "id": "src/api/ComponentEvents.ts::ComponentErrorEvent"
                        }
                    }
                }
            }, {
                "method": "stepCompleteEvent",
                "name": "complete-form-step-event",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "ComponentFormStepCompleteEvent",
                    "resolved": "ComponentFormStepCompleteEvent",
                    "references": {
                        "ComponentFormStepCompleteEvent": {
                            "location": "import",
                            "path": "../../../api/ComponentEvents",
                            "id": "src/api/ComponentEvents.ts::ComponentFormStepCompleteEvent"
                        }
                    }
                }
            }, {
                "method": "submitEvent",
                "name": "submit-event",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "ComponentSubmitEvent",
                    "resolved": "ComponentSubmitEvent",
                    "references": {
                        "ComponentSubmitEvent": {
                            "location": "import",
                            "path": "../../../api/ComponentEvents",
                            "id": "src/api/ComponentEvents.ts::ComponentSubmitEvent"
                        }
                    }
                }
            }];
    }
}
;
