import { h, Host } from "@stencil/core";
import { checkoutStore, onChange } from "../../store/checkout.store";
import JustifiAnalytics from "../../api/Analytics";
import { checkPkgVersion } from "../../utils/check-pkg-version";
import { ComponentErrorCodes, ComponentErrorSeverity } from "../../api";
import { makeCheckoutComplete, makeGetCheckout } from "../../actions/checkout/checkout-actions";
import { CheckoutService } from "../../api/services/checkout.service";
import { insuranceValues, insuranceValuesOn } from "../insurance/insurance-state";
export class CheckoutWrapper {
    constructor() {
        this.savePaymentMethod = false;
    }
    connectedCallback() {
        this.observer = new MutationObserver(() => {
            this.queryFormRefs();
        });
        this.observer.observe(this.hostEl, {
            childList: true,
            subtree: true
        });
        checkoutStore.checkoutId = this.checkoutId;
        const config = {
            authToken: this.authToken,
            checkoutId: this.checkoutId,
            service: new CheckoutService()
        };
        this.getCheckout = makeGetCheckout(config);
        this.completeCheckout = makeCheckoutComplete(config);
        onChange('selectedPaymentMethod', (newValue) => {
            this.paymentMethodChangedEvent.emit(newValue);
        });
    }
    componentWillLoad() {
        this.analytics = new JustifiAnalytics(this);
        checkPkgVersion();
        checkoutStore.authToken = this.authToken;
        this.fetchCheckout();
        // Refresh the checkout data when insurance is added or removed
        insuranceValuesOn('set', (key) => {
            const value = insuranceValues[key];
            if (value !== undefined) {
                this.fetchCheckout();
            }
        });
    }
    componentDidLoad() {
        this.queryFormRefs();
    }
    disconnectedCallback() {
        var _a;
        (_a = this.observer) === null || _a === void 0 ? void 0 : _a.disconnect();
    }
    fetchCheckout() {
        if (this.getCheckout) {
            this.getCheckout({
                onSuccess: ({ checkout }) => {
                    this.updateStore(checkout);
                },
                onError: (error) => {
                    this.errorEvent.emit({
                        message: error.message,
                        errorCode: ComponentErrorCodes.FETCH_ERROR,
                        severity: ComponentErrorSeverity.ERROR,
                    });
                }
            });
        }
    }
    updateStore(checkout) {
        var _a, _b, _c, _d;
        checkoutStore.accountId = checkout.account_id;
        checkoutStore.paymentMethods = checkout.payment_methods;
        checkoutStore.paymentMethodGroupId = checkout.payment_method_group_id;
        checkoutStore.paymentDescription = checkout.payment_description;
        checkoutStore.totalAmount = checkout.total_amount;
        checkoutStore.paymentAmount = checkout.payment_amount;
        checkoutStore.bnplEnabled = checkout.payment_settings.bnpl_payments;
        checkoutStore.bnplProviderClientId = (_a = checkout === null || checkout === void 0 ? void 0 : checkout.bnpl) === null || _a === void 0 ? void 0 : _a.provider_client_id;
        checkoutStore.bnplProviderMode = (_b = checkout === null || checkout === void 0 ? void 0 : checkout.bnpl) === null || _b === void 0 ? void 0 : _b.provider_mode;
        checkoutStore.bnplProviderApiVersion = (_c = checkout === null || checkout === void 0 ? void 0 : checkout.bnpl) === null || _c === void 0 ? void 0 : _c.provider_api_version;
        checkoutStore.bnplProviderCheckoutUrl = (_d = checkout === null || checkout === void 0 ? void 0 : checkout.bnpl) === null || _d === void 0 ? void 0 : _d.provider_checkout_url;
    }
    queryFormRefs() {
        this.paymentMethodFormRef = this.hostEl.querySelector('justifi-card-form, justifi-bank-account-form');
        this.tokenizeComponentRef = this.hostEl.querySelector('justifi-tokenize-payment-method');
        this.billingInformationFormRef = this.hostEl.querySelector('justifi-billing-information-form, justifi-postal-code-form');
        this.insuranceFormRef = this.hostEl.querySelector('justifi-season-interruption-insurance');
        this.sezzlePaymentMethodRef = this.hostEl.querySelector('justifi-sezzle-payment-method');
    }
    async tokenizePaymentMethod(tokenizeArgs) {
        var _a, _b;
        const billingInfoValues = (_b = await ((_a = this.billingInformationFormRef) === null || _a === void 0 ? void 0 : _a.getValues())) !== null && _b !== void 0 ? _b : {};
        const combinedBillingInfo = Object.assign(Object.assign({}, tokenizeArgs), billingInfoValues);
        const paymentMethodMetadata = Object.assign({ accountId: checkoutStore.accountId, payment_method_group_id: undefined }, combinedBillingInfo);
        if (this.savePaymentMethod) {
            paymentMethodMetadata.payment_method_group_id = checkoutStore.paymentMethodGroupId;
        }
        if (this.tokenizeComponentRef) {
            const tokenizeResult = await this.tokenizeComponentRef.tokenizePaymentMethod();
            if (tokenizeResult.error) {
                return tokenizeResult;
            }
        }
        return this.paymentMethodFormRef.tokenize({
            clientId: this.authToken,
            paymentMethodMetadata,
            account: checkoutStore.accountId,
        });
    }
    async getPaymentMethod(submitCheckoutArgs) {
        if (!this.paymentMethodFormRef) {
            return checkoutStore.selectedPaymentMethod;
        }
        const { error, id: token } = await this.tokenizePaymentMethod(submitCheckoutArgs);
        if (error) {
            this.errorEvent.emit({
                errorCode: error.code,
                message: error.message,
                severity: ComponentErrorSeverity.ERROR,
            });
            return undefined;
        }
        return token;
    }
    async validate() {
        var _a, _b;
        const promises = [
            (_a = this.paymentMethodFormRef) === null || _a === void 0 ? void 0 : _a.validate(),
            (_b = this.billingInformationFormRef) === null || _b === void 0 ? void 0 : _b.validate()
        ];
        if (this.insuranceFormRef) {
            promises.push(this.insuranceFormRef.validate());
        }
        const validationResults = await Promise.all(promises);
        return validationResults.every(result => (result === null || result === void 0 ? void 0 : result.isValid) !== false);
    }
    async submitCheckout(submitCheckoutArgs) {
        var _a;
        const isValid = await this.validate();
        if (!isValid) {
            this.errorEvent.emit({
                message: 'Please fill in all required fields.',
                errorCode: ComponentErrorCodes.VALIDATION_ERROR,
                severity: ComponentErrorSeverity.ERROR,
            });
            return;
        }
        let payment;
        if (checkoutStore.selectedPaymentMethod === 'sezzle') {
            const insuranceValidation = this.insuranceFormRef ? await this.insuranceFormRef.validate() : { isValid: true };
            const sezzleResult = await this.sezzlePaymentMethodRef.resolvePaymentMethod(insuranceValidation);
            if (sezzleResult.error) {
                this.errorEvent.emit({
                    message: sezzleResult.error.message,
                    errorCode: ComponentErrorCodes.TOKENIZE_ERROR,
                    severity: ComponentErrorSeverity.ERROR,
                });
                return;
            }
            else if (((_a = sezzleResult.bnpl) === null || _a === void 0 ? void 0 : _a.status) === 'success') {
                payment = {
                    payment_mode: 'bnpl',
                    payment_token: undefined,
                };
            }
        }
        else {
            const paymentMethod = await this.getPaymentMethod(submitCheckoutArgs);
            if (!paymentMethod) {
                this.errorEvent.emit({
                    message: 'Payment method tokenization failed.',
                    errorCode: ComponentErrorCodes.TOKENIZE_ERROR,
                    severity: ComponentErrorSeverity.ERROR,
                });
                return;
            }
            ;
            payment = {
                payment_mode: 'ecom',
                payment_token: paymentMethod,
            };
        }
        this.completeCheckout({
            payment,
            onSuccess: ({ checkout }) => {
                this.submitEvent.emit({
                    checkout,
                    message: 'Checkout completed successfully',
                });
            },
            onError: (error) => {
                this.errorEvent.emit({
                    message: error.message,
                    errorCode: ComponentErrorCodes.COMPLETE_CHECKOUT_ERROR,
                    severity: ComponentErrorSeverity.ERROR,
                });
            },
        });
    }
    async setSelectedPaymentMethod(paymentMethodId) {
        checkoutStore.selectedPaymentMethod = paymentMethodId;
    }
    render() {
        return (h(Host, { key: '6949f68031572493fae333731c78cc07ccd7fafd' }));
    }
    static get is() { return "justifi-modular-checkout"; }
    static get properties() {
        return {
            "authToken": {
                "type": "string",
                "attribute": "auth-token",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "checkoutId": {
                "type": "string",
                "attribute": "checkout-id",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "savePaymentMethod": {
                "type": "boolean",
                "attribute": "save-payment-method",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "false"
            }
        };
    }
    static get events() {
        return [{
                "method": "errorEvent",
                "name": "error-event",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }, {
                "method": "submitEvent",
                "name": "submit-event",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }, {
                "method": "paymentMethodChangedEvent",
                "name": "payment-method-changed",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                }
            }];
    }
    static get methods() {
        return {
            "validate": {
                "complexType": {
                    "signature": "() => Promise<boolean>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<boolean>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            },
            "submitCheckout": {
                "complexType": {
                    "signature": "(submitCheckoutArgs?: BillingFormFields) => Promise<void>",
                    "parameters": [{
                            "name": "submitCheckoutArgs",
                            "type": "BillingFormFields",
                            "docs": ""
                        }],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        },
                        "BillingFormFields": {
                            "location": "import",
                            "path": "../../components",
                            "id": "src/components.d.ts::BillingFormFields"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            },
            "setSelectedPaymentMethod": {
                "complexType": {
                    "signature": "(paymentMethodId: string) => Promise<void>",
                    "parameters": [{
                            "name": "paymentMethodId",
                            "type": "string",
                            "docs": ""
                        }],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            }
        };
    }
    static get elementRef() { return "hostEl"; }
}
