import { h } from "@stencil/core";
import { ComponentErrorCodes, ComponentErrorSeverity, pagingDefaults } from "../../api";
import JustifiAnalytics from "../../api/Analytics";
import { checkPkgVersion } from "../../utils/check-pkg-version";
import { makeGetPayoutTransactions } from "../../actions/payout/get-payout-transactions";
import { PayoutService } from "../../api/services/payout.service";
import { Table } from "../../utils/table";
import { StyledHost, TableEmptyState, TableErrorState, TableLoadingState } from "../../ui-components";
import { defaultColumnsKeys, payoutTransactionTableCells, payoutTransactionTableColumns } from "./payout-transactions-table";
import { configState } from "../config-provider/config-state";
import { TableWrapper, TableComponent, TableHead, TableHeadRow, TableBody, TableRow, TableFoot, TableFootRow, TableFootCell, TableClickActions } from "../../ui-components";
export class PayoutTransactionsList {
    constructor() {
        this.balanceTransactions = [];
        this.isLoading = true;
        this.errorMessage = null;
        this.paging = pagingDefaults;
        this.pagingParams = {};
        this.columns = defaultColumnsKeys;
        this.handleClickPrevious = (beforeCursor) => {
            this.pagingParams = { before_cursor: beforeCursor };
            this.clickEvent.emit({ name: TableClickActions.previous });
        };
        this.handleClickNext = (afterCursor) => {
            this.pagingParams = { after_cursor: afterCursor };
            this.clickEvent.emit({ name: TableClickActions.next });
        };
        this.rowClickHandler = (e) => {
            const clickedPayoutId = e.target.closest('tr').dataset.rowEntityId;
            if (!clickedPayoutId)
                return;
            const transactionData = this.balanceTransactions.find((payout) => payout.id === clickedPayoutId);
            this.clickEvent.emit({ name: TableClickActions.row, data: transactionData });
        };
    }
    componentWillLoad() {
        this.transactionsTable = new Table(this.balanceTransactions, this.columns, payoutTransactionTableColumns, payoutTransactionTableCells);
        checkPkgVersion();
        this.analytics = new JustifiAnalytics(this);
        this.initializeApi();
    }
    disconnectedCallback() {
        var _a;
        (_a = this.analytics) === null || _a === void 0 ? void 0 : _a.cleanup();
    }
    ;
    propChanged() {
        this.initializeApi();
    }
    handleError(code, errorMessage, severity) {
        this.isLoading = false;
        this.errorEvent.emit({ errorCode: code, message: errorMessage, severity });
    }
    initializeApi() {
        if (this.payoutId && this.authToken) {
            const getPayoutTransactions = makeGetPayoutTransactions({
                accountId: configState.accountId,
                authToken: this.authToken,
                service: new PayoutService()
            });
            getPayoutTransactions({
                params: this.requestParams,
                onSuccess: ({ balanceTransactions, pagingInfo }) => {
                    this.balanceTransactions = balanceTransactions;
                    this.paging = pagingInfo;
                    this.transactionsTable.collectionData = this.balanceTransactions;
                },
                onError: ({ error, code, severity }) => {
                    this.errorMessage = error.message;
                    this.handleError(error, code, severity);
                },
                final: () => {
                    this.isLoading = false;
                }
            });
        }
        else {
            this.errorMessage = 'payout-id and auth-token props are required';
            this.handleError(ComponentErrorCodes.MISSING_PROPS, this.errorMessage, ComponentErrorSeverity.ERROR);
        }
    }
    get entityId() {
        return this.balanceTransactions.map((transaction) => transaction.id);
    }
    get showEmptyState() {
        return !this.isLoading && !this.errorMessage && this.transactionsTable.rowData.length < 1;
    }
    get showErrorState() {
        return !this.isLoading && !!this.errorMessage;
    }
    get showRowData() {
        return !this.showEmptyState && !this.showErrorState && !this.isLoading;
    }
    get requestParams() {
        return Object.assign({ payout_id: this.payoutId, limit: '15' }, this.pagingParams);
    }
    render() {
        return (h(StyledHost, { key: 'ed5653473a599f6d6d80aa68730ed999398b0c76' }, h(TableWrapper, { key: 'b3d1d97363d8c5e798583604c7979ba1f9fa2752' }, h(TableComponent, { key: '477d9ff10b92d35b506ef8b78279bb04e4030549' }, h(TableHead, { key: 'c374d061a751b00ec3d7c95e8280653874fabe47' }, h(TableHeadRow, { key: '694753bbfdaaade6c5107100d7f99a4728a59c62' }, this.transactionsTable.columnData.map((column) => column))), h(TableBody, { key: '3e071619a0035b1aec9635ca2476eb279158cb59' }, h(TableLoadingState, { key: '16708244edda538a30bd63af865f9265c3850847', columnSpan: this.transactionsTable.columnKeys.length, isLoading: this.isLoading }), h(TableEmptyState, { key: '150b6e2956f73767d31321fee04d6151818337ce', isEmpty: this.showEmptyState, columnSpan: this.transactionsTable.columnKeys.length }), h(TableErrorState, { key: 'cd5d9464b97567378e07ffd9be1083738d940dc8', columnSpan: this.transactionsTable.columnKeys.length, errorMessage: this.errorMessage }), this.showRowData && this.transactionsTable.rowData.map((data, index) => (h(TableRow, { "data-test-id": "table-row", "data-row-entity-id": this.entityId[index], onClick: this.rowClickHandler }, data)))), this.paging && (h(TableFoot, { key: '83ed7fc0b0c6511de830a3a6880b1c1476a09306' }, h(TableFootRow, { key: '8d636efc57fee42f65165c6aefddab535d30e771' }, h(TableFootCell, { key: '2bed98f16b5b2769c6eb1c5ec8e65c90e1c6fc53', colSpan: this.transactionsTable.columnData.length }, h("pagination-menu", { key: '1ddeac174bc996e08af5e5e060bf8c3303d4c0dd', paging: Object.assign(Object.assign({}, this.paging), { handleClickPrevious: this.handleClickPrevious, handleClickNext: this.handleClickNext }) })))))))));
    }
    static get is() { return "justifi-payout-transactions-list"; }
    static get encapsulation() { return "shadow"; }
    static get properties() {
        return {
            "payoutId": {
                "type": "string",
                "attribute": "payout-id",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "authToken": {
                "type": "string",
                "attribute": "auth-token",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "columns": {
                "type": "string",
                "attribute": "columns",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "defaultColumnsKeys"
            }
        };
    }
    static get states() {
        return {
            "balanceTransactions": {},
            "transactionsTable": {},
            "isLoading": {},
            "errorMessage": {},
            "paging": {},
            "pagingParams": {}
        };
    }
    static get events() {
        return [{
                "method": "clickEvent",
                "name": "click-event",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "ComponentClickEvent",
                    "resolved": "ComponentClickEvent",
                    "references": {
                        "ComponentClickEvent": {
                            "location": "import",
                            "path": "../../api",
                            "id": "src/api/index.ts::ComponentClickEvent"
                        }
                    }
                }
            }, {
                "method": "errorEvent",
                "name": "error-event",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "ComponentErrorEvent",
                    "resolved": "ComponentErrorEvent",
                    "references": {
                        "ComponentErrorEvent": {
                            "location": "import",
                            "path": "../../api",
                            "id": "src/api/index.ts::ComponentErrorEvent"
                        }
                    }
                }
            }];
    }
    static get watchers() {
        return [{
                "propName": "pagingParams",
                "methodName": "propChanged"
            }, {
                "propName": "payoutId",
                "methodName": "propChanged"
            }, {
                "propName": "authToken",
                "methodName": "propChanged"
            }];
    }
}
