import { proxyCustomElement, HTMLElement, createEvent, h } from '@stencil/core/internal/client';
import { A as Api } from './Api.js';
import { C as ComponentErrorSeverity, a as ComponentErrorCodes } from './Refund.js';
import './Pagination.js';
import './Business.js';
import { g as getErrorCode, a as getErrorMessage } from './utils4.js';
import { c as checkPkgVersion, J as JustifiAnalytics } from './Analytics.js';
import { c as checkoutStore } from './checkout.store.js';
import { d as defineCustomElement$4 } from './form-control-help-text2.js';
import { d as defineCustomElement$3 } from './form-control-radio2.js';
import { d as defineCustomElement$2 } from './season-interruption-insurance-core.js';

const api = Api();
class InsuranceService {
    async fetchQuote(authToken, payload) {
        const endpoint = 'insurance/quotes';
        const body = payload;
        return api.post({ endpoint, body, authToken });
    }
    async toggleCoverage(authToken, quoteId, payload) {
        const endpoint = `insurance/quotes/${quoteId}/toggle`;
        const body = payload;
        return api.post({ endpoint, body, authToken });
    }
}

const makeGetQuote = ({ authToken, service }) => async ({ payload, onSuccess, onError }) => {
    var _a;
    try {
        const response = await service.fetchQuote(authToken, payload);
        if (!response.error) {
            const quote = response.data;
            onSuccess({ quote });
        }
        else {
            const responseError = getErrorMessage(response.error);
            const code = getErrorCode((_a = response.error) === null || _a === void 0 ? void 0 : _a.code);
            return onError({
                error: responseError,
                code,
                severity: ComponentErrorSeverity.ERROR,
            });
        }
    }
    catch (error) {
        const code = getErrorCode(error === null || error === void 0 ? void 0 : error.code);
        return onError({
            error: error.message || error,
            code,
            severity: ComponentErrorSeverity.ERROR,
        });
    }
};
const makeToggleCoverage = ({ authToken, service }) => async ({ quoteId, payload, onSuccess, onError }) => {
    var _a;
    try {
        const response = await service.toggleCoverage(authToken, quoteId, payload);
        if (!response.error) {
            const quote = response.data;
            onSuccess({ quote });
        }
        else {
            const responseError = getErrorMessage(response.error);
            const code = getErrorCode((_a = response.error) === null || _a === void 0 ? void 0 : _a.code);
            return onError({
                error: responseError,
                code,
                severity: ComponentErrorSeverity.ERROR,
            });
        }
    }
    catch (error) {
        const code = getErrorCode(error === null || error === void 0 ? void 0 : error.code);
        return onError({
            error: error.message || error,
            code,
            severity: ComponentErrorSeverity.ERROR,
        });
    }
};

const SeasonInterruptionInsurance = /*@__PURE__*/ proxyCustomElement(class SeasonInterruptionInsurance extends HTMLElement {
    constructor() {
        super();
        this.__registerHost();
        this.errorEvent = createEvent(this, "error-event", 7);
    }
    async validate() {
        return this.seasonInterruptionCoreRef.validate();
    }
    componentWillLoad() {
        checkPkgVersion();
        this.analytics = new JustifiAnalytics(this);
        this.initializeServiceMethods();
    }
    disconnectedCallback() {
        var _a;
        (_a = this.analytics) === null || _a === void 0 ? void 0 : _a.cleanup();
    }
    initializeServiceMethods() {
        const authToken = this.authToken || checkoutStore.authToken;
        if (!authToken) {
            this.errorEvent.emit({
                errorCode: ComponentErrorCodes.MISSING_PROPS,
                message: 'Missing authToken',
                severity: ComponentErrorSeverity.ERROR,
            });
            return;
        }
        const service = new InsuranceService();
        const config = { authToken, service };
        this.getQuote = makeGetQuote(config);
        this.toggleCoverage = makeToggleCoverage(config);
    }
    render() {
        return (h("justifi-season-interruption-insurance-core", { key: '394fc5030f618621fe87b8760b9d0847fa3a496c', ref: (el) => (this.seasonInterruptionCoreRef = el), "checkout-id": this.checkoutId || checkoutStore.checkoutId, getQuote: this.getQuote, toggleCoverage: this.toggleCoverage, "primary-identity-first-name": this.primaryIdentityFirstName, "primary-identity-last-name": this.primaryIdentityLastName, "primary-identity-state": this.primaryIdentityState, "primary-identity-postal-code": this.primaryIdentityPostalCode, "primary-identity-country": this.primaryIdentityCountry, "primary-identity-email-address": this.primaryIdentityEmailAddress, "policy-attributes-insurable-amount": this.policyAttributesInsurableAmount, "policy-attributes-start-date": this.policyAttributesStartDate, "policy-attributes-end-date": this.policyAttributesEndDate, "covered-identity-first-name": this.coveredIdentityFirstName, "covered-identity-last-name": this.coveredIdentityLastName }));
    }
}, [0, "justifi-season-interruption-insurance", {
        "authToken": [1, "auth-token"],
        "checkoutId": [1, "checkout-id"],
        "primaryIdentityFirstName": [1, "primary-identity-first-name"],
        "primaryIdentityLastName": [1, "primary-identity-last-name"],
        "primaryIdentityState": [1, "primary-identity-state"],
        "primaryIdentityPostalCode": [1, "primary-identity-postal-code"],
        "primaryIdentityCountry": [1, "primary-identity-country"],
        "primaryIdentityEmailAddress": [1, "primary-identity-email-address"],
        "policyAttributesInsurableAmount": [2, "policy-attributes-insurable-amount"],
        "policyAttributesStartDate": [1, "policy-attributes-start-date"],
        "policyAttributesEndDate": [1, "policy-attributes-end-date"],
        "coveredIdentityFirstName": [1, "covered-identity-first-name"],
        "coveredIdentityLastName": [1, "covered-identity-last-name"],
        "getQuote": [32],
        "toggleCoverage": [32],
        "quote": [32],
        "validate": [64]
    }]);
function defineCustomElement$1() {
    if (typeof customElements === "undefined") {
        return;
    }
    const components = ["justifi-season-interruption-insurance", "form-control-help-text", "form-control-radio", "justifi-season-interruption-insurance-core"];
    components.forEach(tagName => { switch (tagName) {
        case "justifi-season-interruption-insurance":
            if (!customElements.get(tagName)) {
                customElements.define(tagName, SeasonInterruptionInsurance);
            }
            break;
        case "form-control-help-text":
            if (!customElements.get(tagName)) {
                defineCustomElement$4();
            }
            break;
        case "form-control-radio":
            if (!customElements.get(tagName)) {
                defineCustomElement$3();
            }
            break;
        case "justifi-season-interruption-insurance-core":
            if (!customElements.get(tagName)) {
                defineCustomElement$2();
            }
            break;
    } });
}
defineCustomElement$1();

const JustifiSeasonInterruptionInsurance = SeasonInterruptionInsurance;
const defineCustomElement = defineCustomElement$1;

export { JustifiSeasonInterruptionInsurance, defineCustomElement };
