import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { c as checkoutStore, o as onChange } from './checkout.store.js';
import { J as JustifiAnalytics, c as checkPkgVersion } from './Analytics.js';
import './config-state.js';
import './dinero.js';
import { C as ComponentErrorSeverity, a as ComponentErrorCodes } from './Refund.js';
import './Pagination.js';
import './Business.js';
import { g as getErrorCode, a as getErrorMessage } from './utils4.js';
import { C as CheckoutService } from './checkout.service.js';
import { i as insuranceValuesOn, a as insuranceValues } from './insurance-state.js';

const makeGetCheckout = ({ authToken, checkoutId, service }) => async ({ onSuccess, onError }) => {
    var _a;
    try {
        const response = await service.fetchCheckout(authToken, checkoutId);
        if (!response.error) {
            const checkout = response.data;
            onSuccess({ checkout });
        }
        else {
            const responseError = getErrorMessage(response.error);
            const code = getErrorCode((_a = response.error) === null || _a === void 0 ? void 0 : _a.code);
            return onError({
                error: responseError,
                code,
                severity: ComponentErrorSeverity.ERROR,
            });
        }
    }
    catch (error) {
        const code = getErrorCode(error === null || error === void 0 ? void 0 : error.code);
        return onError({
            error: error.message || error,
            code,
            severity: ComponentErrorSeverity.ERROR,
        });
    }
};
const makeCheckoutComplete = ({ authToken, checkoutId, service }) => async ({ payment, onSuccess, onError }) => {
    var _a;
    try {
        const response = await service.complete(authToken, checkoutId, payment);
        if (!response.error) {
            onSuccess(response);
        }
        else {
            const responseError = getErrorMessage(response.error);
            const code = getErrorCode((_a = response.error) === null || _a === void 0 ? void 0 : _a.code);
            return onError({
                error: responseError,
                code,
                severity: ComponentErrorSeverity.ERROR,
            });
        }
    }
    catch (error) {
        const code = getErrorCode(error === null || error === void 0 ? void 0 : error.code);
        return onError({
            error: error.message || error,
            code,
            severity: ComponentErrorSeverity.ERROR,
        });
    }
};

const CheckoutWrapper = /*@__PURE__*/ proxyCustomElement(class CheckoutWrapper extends HTMLElement {
    constructor() {
        super();
        this.__registerHost();
        this.errorEvent = createEvent(this, "error-event", 7);
        this.submitEvent = createEvent(this, "submit-event", 7);
        this.paymentMethodChangedEvent = createEvent(this, "payment-method-changed", 7);
        this.savePaymentMethod = false;
    }
    connectedCallback() {
        this.observer = new MutationObserver(() => {
            this.queryFormRefs();
        });
        this.observer.observe(this.hostEl, {
            childList: true,
            subtree: true
        });
        checkoutStore.checkoutId = this.checkoutId;
        const config = {
            authToken: this.authToken,
            checkoutId: this.checkoutId,
            service: new CheckoutService()
        };
        this.getCheckout = makeGetCheckout(config);
        this.completeCheckout = makeCheckoutComplete(config);
        onChange('selectedPaymentMethod', (newValue) => {
            this.paymentMethodChangedEvent.emit(newValue);
        });
    }
    componentWillLoad() {
        this.analytics = new JustifiAnalytics(this);
        checkPkgVersion();
        checkoutStore.authToken = this.authToken;
        this.fetchCheckout();
        // Refresh the checkout data when insurance is added or removed
        insuranceValuesOn('set', (key) => {
            const value = insuranceValues[key];
            if (value !== undefined) {
                this.fetchCheckout();
            }
        });
    }
    componentDidLoad() {
        this.queryFormRefs();
    }
    disconnectedCallback() {
        var _a;
        (_a = this.observer) === null || _a === void 0 ? void 0 : _a.disconnect();
    }
    fetchCheckout() {
        if (this.getCheckout) {
            this.getCheckout({
                onSuccess: ({ checkout }) => {
                    this.updateStore(checkout);
                },
                onError: (error) => {
                    this.errorEvent.emit({
                        message: error.message,
                        errorCode: ComponentErrorCodes.FETCH_ERROR,
                        severity: ComponentErrorSeverity.ERROR,
                    });
                }
            });
        }
    }
    updateStore(checkout) {
        var _a, _b, _c, _d;
        checkoutStore.accountId = checkout.account_id;
        checkoutStore.paymentMethods = checkout.payment_methods;
        checkoutStore.paymentMethodGroupId = checkout.payment_method_group_id;
        checkoutStore.paymentDescription = checkout.payment_description;
        checkoutStore.totalAmount = checkout.total_amount;
        checkoutStore.paymentAmount = checkout.payment_amount;
        checkoutStore.bnplEnabled = checkout.payment_settings.bnpl_payments;
        checkoutStore.bnplProviderClientId = (_a = checkout === null || checkout === void 0 ? void 0 : checkout.bnpl) === null || _a === void 0 ? void 0 : _a.provider_client_id;
        checkoutStore.bnplProviderMode = (_b = checkout === null || checkout === void 0 ? void 0 : checkout.bnpl) === null || _b === void 0 ? void 0 : _b.provider_mode;
        checkoutStore.bnplProviderApiVersion = (_c = checkout === null || checkout === void 0 ? void 0 : checkout.bnpl) === null || _c === void 0 ? void 0 : _c.provider_api_version;
        checkoutStore.bnplProviderCheckoutUrl = (_d = checkout === null || checkout === void 0 ? void 0 : checkout.bnpl) === null || _d === void 0 ? void 0 : _d.provider_checkout_url;
    }
    queryFormRefs() {
        this.paymentMethodFormRef = this.hostEl.querySelector('justifi-card-form, justifi-bank-account-form');
        this.tokenizeComponentRef = this.hostEl.querySelector('justifi-tokenize-payment-method');
        this.billingInformationFormRef = this.hostEl.querySelector('justifi-billing-information-form, justifi-postal-code-form');
        this.insuranceFormRef = this.hostEl.querySelector('justifi-season-interruption-insurance');
        this.sezzlePaymentMethodRef = this.hostEl.querySelector('justifi-sezzle-payment-method');
    }
    async tokenizePaymentMethod(tokenizeArgs) {
        var _a, _b;
        const billingInfoValues = (_b = await ((_a = this.billingInformationFormRef) === null || _a === void 0 ? void 0 : _a.getValues())) !== null && _b !== void 0 ? _b : {};
        const combinedBillingInfo = Object.assign(Object.assign({}, tokenizeArgs), billingInfoValues);
        const paymentMethodMetadata = Object.assign({ accountId: checkoutStore.accountId, payment_method_group_id: undefined }, combinedBillingInfo);
        if (this.savePaymentMethod) {
            paymentMethodMetadata.payment_method_group_id = checkoutStore.paymentMethodGroupId;
        }
        if (this.tokenizeComponentRef) {
            const tokenizeResult = await this.tokenizeComponentRef.tokenizePaymentMethod();
            if (tokenizeResult.error) {
                return tokenizeResult;
            }
        }
        return this.paymentMethodFormRef.tokenize({
            clientId: this.authToken,
            paymentMethodMetadata,
            account: checkoutStore.accountId,
        });
    }
    async getPaymentMethod(submitCheckoutArgs) {
        if (!this.paymentMethodFormRef) {
            return checkoutStore.selectedPaymentMethod;
        }
        const { error, id: token } = await this.tokenizePaymentMethod(submitCheckoutArgs);
        if (error) {
            this.errorEvent.emit({
                errorCode: error.code,
                message: error.message,
                severity: ComponentErrorSeverity.ERROR,
            });
            return undefined;
        }
        return token;
    }
    async validate() {
        var _a, _b;
        const promises = [
            (_a = this.paymentMethodFormRef) === null || _a === void 0 ? void 0 : _a.validate(),
            (_b = this.billingInformationFormRef) === null || _b === void 0 ? void 0 : _b.validate()
        ];
        if (this.insuranceFormRef) {
            promises.push(this.insuranceFormRef.validate());
        }
        const validationResults = await Promise.all(promises);
        return validationResults.every(result => (result === null || result === void 0 ? void 0 : result.isValid) !== false);
    }
    async submitCheckout(submitCheckoutArgs) {
        var _a;
        const isValid = await this.validate();
        if (!isValid) {
            this.errorEvent.emit({
                message: 'Please fill in all required fields.',
                errorCode: ComponentErrorCodes.VALIDATION_ERROR,
                severity: ComponentErrorSeverity.ERROR,
            });
            return;
        }
        let payment;
        if (checkoutStore.selectedPaymentMethod === 'sezzle') {
            const insuranceValidation = this.insuranceFormRef ? await this.insuranceFormRef.validate() : { isValid: true };
            const sezzleResult = await this.sezzlePaymentMethodRef.resolvePaymentMethod(insuranceValidation);
            if (sezzleResult.error) {
                this.errorEvent.emit({
                    message: sezzleResult.error.message,
                    errorCode: ComponentErrorCodes.TOKENIZE_ERROR,
                    severity: ComponentErrorSeverity.ERROR,
                });
                return;
            }
            else if (((_a = sezzleResult.bnpl) === null || _a === void 0 ? void 0 : _a.status) === 'success') {
                payment = {
                    payment_mode: 'bnpl',
                    payment_token: undefined,
                };
            }
        }
        else {
            const paymentMethod = await this.getPaymentMethod(submitCheckoutArgs);
            if (!paymentMethod) {
                this.errorEvent.emit({
                    message: 'Payment method tokenization failed.',
                    errorCode: ComponentErrorCodes.TOKENIZE_ERROR,
                    severity: ComponentErrorSeverity.ERROR,
                });
                return;
            }
            payment = {
                payment_mode: 'ecom',
                payment_token: paymentMethod,
            };
        }
        this.completeCheckout({
            payment,
            onSuccess: ({ checkout }) => {
                this.submitEvent.emit({
                    checkout,
                    message: 'Checkout completed successfully',
                });
            },
            onError: (error) => {
                this.errorEvent.emit({
                    message: error.message,
                    errorCode: ComponentErrorCodes.COMPLETE_CHECKOUT_ERROR,
                    severity: ComponentErrorSeverity.ERROR,
                });
            },
        });
    }
    async setSelectedPaymentMethod(paymentMethodId) {
        checkoutStore.selectedPaymentMethod = paymentMethodId;
    }
    render() {
        return (h(Host, { key: '6949f68031572493fae333731c78cc07ccd7fafd' }));
    }
    get hostEl() { return this; }
}, [0, "justifi-modular-checkout", {
        "authToken": [1, "auth-token"],
        "checkoutId": [1, "checkout-id"],
        "savePaymentMethod": [4, "save-payment-method"],
        "validate": [64],
        "submitCheckout": [64],
        "setSelectedPaymentMethod": [64]
    }]);
function defineCustomElement() {
    if (typeof customElements === "undefined") {
        return;
    }
    const components = ["justifi-modular-checkout"];
    components.forEach(tagName => { switch (tagName) {
        case "justifi-modular-checkout":
            if (!customElements.get(tagName)) {
                customElements.define(tagName, CheckoutWrapper);
            }
            break;
    } });
}
defineCustomElement();

export { CheckoutWrapper as C, defineCustomElement as d };
