import { h, proxyCustomElement, HTMLElement, createEvent } from '@stencil/core/internal/client';
import { T as Table, a as TableWrapper, b as TableComponent, c as TableHead, d as TableHeadRow, e as TableBody, f as TableLoadingState, g as TableEmptyState, h as TableErrorState, i as TableRow, j as TableFoot, k as TableFootRow, l as TableFootCell } from './table.js';
import './details.js';
import { T as TableClickActions } from './event-types.js';
import './justifi-radio-list-item2.js';
import './config-state.js';
import './dinero.js';
import { e as TerminalOrderType } from './Refund.js';
import { p as pagingDefaults } from './Pagination.js';
import './Business.js';
import { v as tableHeadCell, a8 as getAlternateTableCellPart } from './parts.js';
import { e as convertToLocal } from './utils2.js';
import { B as Badge, a as BadgeVariant } from './badge.js';
import { o as onQueryParamsChange, g as getRequestParams } from './terminal-orders-list-params-state.js';
import { d as defineCustomElement$1 } from './pagination-menu2.js';

const MapTerminalOrderStatusToBadge = (status) => {
    const statusToBadgeProps = {
        submitted: {
            variant: BadgeVariant.PRIMARY,
            title: 'This order is submitted',
            text: 'Submitted',
        },
        completed: {
            variant: BadgeVariant.SUCCESS,
            title: 'This order has been completed',
            text: 'Completed',
        },
        created: {
            variant: BadgeVariant.SECONDARY,
            title: 'This order is created',
            text: 'Created',
        },
        in_progress: {
            variant: BadgeVariant.PRIMARY,
            title: 'This order is in progress',
            text: 'In Progress',
        },
        on_hold: {
            variant: BadgeVariant.WARNING,
            title: 'This order is on hold',
            text: 'On Hold',
        },
        canceled: {
            variant: BadgeVariant.DANGER,
            title: 'This order is canceled',
            text: 'Canceled',
        }
    };
    const badgeProps = statusToBadgeProps[status];
    return h(Badge, Object.assign({}, badgeProps));
};

const defaultColumnsKeys = 'created_at,updated_at,order_status,quantity';
const terminalOrdersTableColumns = {
    created_at: () => (h("th", { part: tableHeadCell, scope: "col", title: "The date and time the order was created" }, "Created At")),
    updated_at: () => (h("th", { part: tableHeadCell, scope: "col", title: "The date and time the order was last updated" }, "Updated At")),
    order_status: () => (h("th", { part: tableHeadCell, scope: "col", title: "The current status of the order" }, "Order Status")),
    business_id: () => (h("th", { part: tableHeadCell, scope: "col", title: "The business ID associated with the order" }, "Business ID")),
    company_name: () => (h("th", { part: tableHeadCell, scope: "col", title: "The business name associated with the order" }, "Business Name")),
    provider: () => (h("th", { part: tableHeadCell, scope: "col", title: "The provider associated with the order" }, "Provider")),
    order_type: () => (h("th", { part: tableHeadCell, scope: "col", title: "The type of order" }, "Order Type")),
    quantity: () => (h("th", { part: tableHeadCell, scope: "col", title: "The number of terminals associated with the order" }, "Order Quantity")),
};
const terminalOrdersTableCells = {
    created_at: (terminalOrder, index) => (h("td", { part: getAlternateTableCellPart(index) },
        h("div", { class: "fw-bold" }, convertToLocal(terminalOrder.created_at, { showDisplayDate: true })),
        h("div", { class: "fw-bold" }, convertToLocal(terminalOrder.created_at, { showTime: true })))),
    updated_at: (terminalOrder, index) => (h("td", { part: getAlternateTableCellPart(index) },
        h("div", { class: "fw-bold" }, convertToLocal(terminalOrder.updated_at, { showDisplayDate: true })),
        h("div", { class: "fw-bold" }, convertToLocal(terminalOrder.updated_at, { showTime: true })))),
    order_status: (terminalOrder, index) => (h("td", { part: getAlternateTableCellPart(index) }, MapTerminalOrderStatusToBadge(terminalOrder.order_status))),
    business_id: (terminalOrder, index) => (h("td", { part: getAlternateTableCellPart(index) }, terminalOrder.business_id)),
    company_name: (terminalOrder, index) => (h("td", { part: getAlternateTableCellPart(index) }, terminalOrder.company_name)),
    provider: (terminalOrder, index) => (h("td", { part: getAlternateTableCellPart(index) }, terminalOrder.provider)),
    order_type: (terminalOrder, index) => {
        let orderType;
        if (terminalOrder.order_type === TerminalOrderType.boardingOnly) {
            orderType = 'Boarding Only';
        }
        else if (terminalOrder.order_type === TerminalOrderType.boardingShipping) {
            orderType = 'Boarding & Shipping';
        }
        return (h("td", { part: getAlternateTableCellPart(index) }, orderType));
    },
    quantity: (terminalOrder, index) => (h("td", { part: getAlternateTableCellPart(index) }, terminalOrder.terminals.length)),
};

const TerminalOrdersListCore = /*@__PURE__*/ proxyCustomElement(class TerminalOrdersListCore extends HTMLElement {
    constructor() {
        super();
        this.__registerHost();
        this.clickEvent = createEvent(this, "click-event", 7);
        this.errorEvent = createEvent(this, "error-event", 7);
        this.terminalOrders = [];
        this.loading = true;
        this.paging = pagingDefaults;
        this.pagingParams = {};
        this.handleClickPrevious = (beforeCursor) => {
            this.pagingParams = { before_cursor: beforeCursor };
            this.clickEvent.emit({ name: TableClickActions.previous });
        };
        this.handleClickNext = (afterCursor) => {
            this.pagingParams = { after_cursor: afterCursor };
            this.clickEvent.emit({ name: TableClickActions.next });
        };
        this.rowClickHandler = (e) => {
            const clickedOrderId = e.target.closest('tr').dataset.rowEntityId;
            if (!clickedOrderId)
                return;
            const orderData = this.terminalOrders.find((order) => order.id === clickedOrderId);
            this.clickEvent.emit({ name: TableClickActions.row, data: orderData });
        };
    }
    updateOnPropChange() {
        this.fetchData();
    }
    componentWillLoad() {
        this.terminalOrdersTable = new Table(this.terminalOrders, this.columns, terminalOrdersTableColumns, terminalOrdersTableCells);
        if (this.getTerminalOrders) {
            this.fetchData();
        }
        onQueryParamsChange('set', () => {
            this.pagingParams = {};
        });
        onQueryParamsChange('reset', () => {
            this.pagingParams = {};
            this.errorMessage = '';
        });
    }
    fetchData() {
        this.loading = true;
        this.getTerminalOrders({
            params: this.terminalOrderParams,
            onSuccess: async ({ terminalOrders, pagingInfo }) => {
                this.terminalOrders = terminalOrders;
                this.paging = pagingInfo;
                this.terminalOrdersTable.collectionData = this.terminalOrders;
            },
            onError: ({ error, code, severity }) => {
                this.errorMessage = error;
                this.errorEvent.emit({
                    errorCode: code,
                    message: error,
                    severity,
                });
            },
            final: () => { this.loading = false; },
        });
    }
    get terminalOrderParams() {
        const requestParams = getRequestParams();
        const params = Object.assign(Object.assign({}, requestParams), this.pagingParams);
        return params;
    }
    get entityId() {
        return this.terminalOrders.map((terminalOrder) => terminalOrder.id);
    }
    get showEmptyState() {
        return !this.loading && !this.errorMessage && this.terminalOrdersTable.rowData.length < 1;
    }
    get showErrorState() {
        return !this.loading && !!this.errorMessage;
    }
    get showRowData() {
        return !this.showEmptyState && !this.showErrorState && !this.loading;
    }
    render() {
        return (h(TableWrapper, { key: '85a1a02fb87d8582cb58716fceab3037f2b2db59' }, h(TableComponent, { key: '16d1f186acc00b28437ebb5608a69b6b69fa7914' }, h(TableHead, { key: 'dce803614dd4c1fd7844aeeb9539cfa32368b277' }, h(TableHeadRow, { key: 'f8c81e2b7e35064e8f2699366045fdf63d0b6a25' }, this.terminalOrdersTable.columnData.map((column) => column))), h(TableBody, { key: 'f1d27f78f5985b0e7b0e24bc370e787d08d7d441' }, h(TableLoadingState, { key: '036e603cd4ba2785b931f9c101c73bda204e0c3b', columnSpan: this.terminalOrdersTable.columnKeys.length, isLoading: this.loading }), h(TableEmptyState, { key: '7b7e58eebc3644e05e989e6593cc0b995f52f3fc', isEmpty: this.showEmptyState, columnSpan: this.terminalOrdersTable.columnKeys.length }), h(TableErrorState, { key: '2ba2fa702cc279cfb4bb6116aef5f99bcc6cdeea', columnSpan: this.terminalOrdersTable.columnKeys.length, errorMessage: this.errorMessage }), this.showRowData && this.terminalOrdersTable.rowData.map((data, index) => (h(TableRow, { "data-test-id": "table-row", "data-row-entity-id": this.entityId[index], onClick: this.rowClickHandler }, data)))), this.paging && (h(TableFoot, { key: '8448f124fd6d5cb82983af8b163532e067d44e7f' }, h(TableFootRow, { key: '3172efc0062b99a291ef945c900321d4db112dda' }, h(TableFootCell, { key: '690c7414a1a3071a0d662c14ebe86b5f771da10a', colSpan: this.terminalOrdersTable.columnData.length }, h("pagination-menu", { key: '8034a4c00f149c1b720db9c78b91e6ea635bd86e', paging: Object.assign(Object.assign({}, this.paging), { handleClickPrevious: this.handleClickPrevious, handleClickNext: this.handleClickNext }) }))))))));
    }
    static get watchers() { return {
        "pagingParams": ["updateOnPropChange"],
        "getTerminalOrders": ["updateOnPropChange"],
        "columns": ["updateOnPropChange"]
    }; }
}, [0, "terminal-orders-list-core", {
        "getTerminalOrders": [16, "get-terminal-orders"],
        "columns": [1],
        "terminalOrders": [32],
        "terminalOrdersTable": [32],
        "loading": [32],
        "errorMessage": [32],
        "paging": [32],
        "pagingParams": [32]
    }, undefined, {
        "pagingParams": ["updateOnPropChange"],
        "getTerminalOrders": ["updateOnPropChange"],
        "columns": ["updateOnPropChange"]
    }]);
function defineCustomElement() {
    if (typeof customElements === "undefined") {
        return;
    }
    const components = ["terminal-orders-list-core", "pagination-menu"];
    components.forEach(tagName => { switch (tagName) {
        case "terminal-orders-list-core":
            if (!customElements.get(tagName)) {
                customElements.define(tagName, TerminalOrdersListCore);
            }
            break;
        case "pagination-menu":
            if (!customElements.get(tagName)) {
                defineCustomElement$1();
            }
            break;
    } });
}
defineCustomElement();

export { TerminalOrdersListCore as T, defaultColumnsKeys as a, defineCustomElement as d };
