import { h, proxyCustomElement, HTMLElement, createEvent } from '@stencil/core/internal/client';
import './config-state.js';
import './dinero.js';
import './Refund.js';
import { p as pagingDefaults } from './Pagination.js';
import './Business.js';
import './details.js';
import { T as Table, a as TableWrapper, b as TableComponent, c as TableHead, d as TableHeadRow, e as TableBody, f as TableLoadingState, g as TableEmptyState, h as TableErrorState, i as TableRow, j as TableFoot, k as TableFootRow, l as TableFootCell } from './table.js';
import { T as TableClickActions } from './event-types.js';
import './justifi-radio-list-item2.js';
import { B as Badge, a as BadgeVariant } from './badge.js';
import { v as tableHeadCell, a8 as getAlternateTableCellPart } from './parts.js';
import { k as capitalFirstLetter } from './utils2.js';
import { o as onQueryParamsChange, g as getRequestParams } from './terminals-list-params-state.js';
import { d as defineCustomElement$1 } from './pagination-menu2.js';

const MapTerminalStatusToBadge = (status) => {
    const statusToBadgeProps = {
        connected: {
            variant: BadgeVariant.SUCCESS,
            title: 'This terminal is connected',
            text: 'Connected',
        },
        disconnected: {
            variant: BadgeVariant.DANGER,
            title: 'This terminal is disconnected',
            text: 'Disconnected',
        },
        unknown: {
            variant: BadgeVariant.SECONDARY,
            title: 'The status of this terminal is unknown',
            text: 'Unknown',
        },
        pending_configuration: {
            variant: BadgeVariant.WARNING,
            title: 'This terminal is pending configuration',
            text: 'Pending configuration',
        },
    };
    const badgeProps = statusToBadgeProps[status];
    return h(Badge, Object.assign({}, badgeProps));
};

const defaultColumnsKeys = 'nickname,provider_serial_number,status';
const terminalTableColumns = {
    nickname: () => (h("th", { part: tableHeadCell, scope: "col", title: "The nickname associated with the terminal" }, "Nickname")),
    model_name: () => (h("th", { part: tableHeadCell, scope: "col", title: "The model name of the terminal" }, "Model Name")),
    id: () => (h("th", { part: tableHeadCell, scope: "col", title: "The ID of the terminal" }, "ID")),
    provider: () => (h("th", { part: tableHeadCell, scope: "col", title: "The provider of the terminal" }, "Provider")),
    provider_serial_number: () => (h("th", { part: tableHeadCell, scope: "col", title: "The serial number of the terminal" }, "Serial Number")),
    device_id: () => (h("th", { part: tableHeadCell, scope: "col", title: "The device/provider ID of the terminal" }, "Device ID")),
    sub_account_name: () => (h("th", { part: tableHeadCell, scope: "col", title: "The sub account associated with the terminal" }, "Sub Account")),
    status: () => (h("th", { part: tableHeadCell, scope: "col", title: "The current status of each terminal" }, "Status")),
};
const terminalTableCells = {
    nickname: (terminal, index) => (h("td", { part: getAlternateTableCellPart(index) }, terminal.nickname)),
    model_name: (terminal, index) => (h("td", { part: getAlternateTableCellPart(index) }, terminal.model_name)),
    id: (terminal, index) => (h("td", { part: getAlternateTableCellPart(index) }, terminal.id)),
    provider: (terminal, index) => (h("td", { part: getAlternateTableCellPart(index) }, capitalFirstLetter(terminal.provider))),
    provider_serial_number: (terminal, index) => (h("td", { part: getAlternateTableCellPart(index) }, terminal.provider_serial_number)),
    device_id: (terminal, index) => (h("td", { part: getAlternateTableCellPart(index) }, terminal.provider_id)),
    sub_account_name: (terminal, index) => (h("td", { part: getAlternateTableCellPart(index) }, terminal.sub_account_name)),
    status: (terminal, index) => (h("td", { part: getAlternateTableCellPart(index) }, MapTerminalStatusToBadge(terminal.status))),
};

const TerminalsListCore = /*@__PURE__*/ proxyCustomElement(class TerminalsListCore extends HTMLElement {
    constructor() {
        super();
        this.__registerHost();
        this.clickEvent = createEvent(this, "click-event", 7);
        this.errorEvent = createEvent(this, "error-event", 7);
        this.terminals = [];
        this.subAccounts = [];
        this.loading = true;
        this.paging = pagingDefaults;
        this.pagingParams = {};
        this.handleClickPrevious = (beforeCursor) => {
            this.pagingParams = { before_cursor: beforeCursor };
            this.clickEvent.emit({ name: TableClickActions.previous });
        };
        this.handleClickNext = (afterCursor) => {
            this.pagingParams = { after_cursor: afterCursor };
            this.clickEvent.emit({ name: TableClickActions.next });
        };
        this.rowClickHandler = (e) => {
            const clickedTerminalID = e.target.closest('tr').dataset.rowEntityId;
            if (!clickedTerminalID)
                return;
            const terminalData = this.terminals.find((terminal) => terminal.id === clickedTerminalID);
            this.clickEvent.emit({ name: TableClickActions.row, data: terminalData });
        };
    }
    updateOnPropChange() {
        this.fetchData();
    }
    componentWillLoad() {
        this.terminalsTable = new Table(this.terminals, this.columns, terminalTableColumns, terminalTableCells);
        if (this.getTerminals && this.getSubAccounts) {
            this.fetchData();
        }
        onQueryParamsChange('set', () => {
            this.pagingParams = {};
        });
        onQueryParamsChange('reset', () => {
            this.pagingParams = {};
            this.errorMessage = '';
        });
    }
    fetchData() {
        this.loading = true;
        this.getTerminals({
            params: this.terminalParams,
            onSuccess: async ({ terminals, pagingInfo }) => {
                this.terminals = terminals;
                this.paging = pagingInfo;
                this.terminalsTable.collectionData = this.terminals;
                const shouldFetchSubAccounts = this.terminalsTable.columnKeys.includes('sub_account_name');
                if (shouldFetchSubAccounts) {
                    await this.fetchSubAccounts();
                }
                else {
                    this.loading = false;
                }
            },
            onError: ({ error, code, severity }) => {
                this.errorMessage = error;
                this.errorEvent.emit({
                    errorCode: code,
                    message: error,
                    severity,
                });
                this.loading = false;
            },
        });
    }
    async fetchSubAccounts() {
        this.getSubAccounts({
            params: this.subAccountParams,
            onSuccess: ({ subAccounts }) => {
                this.subAccounts = subAccounts;
                this.terminals = this.terminals.map((terminal) => {
                    var _a;
                    terminal.sub_account_name = (_a = this.subAccounts.find((subAccount) => subAccount.id === terminal.account_id)) === null || _a === void 0 ? void 0 : _a.name;
                    return terminal;
                });
                this.loading = false;
            },
            onError: ({ error, code, severity }) => {
                this.errorMessage = error;
                this.errorEvent.emit({
                    errorCode: code,
                    message: error,
                    severity,
                });
                this.loading = false;
            },
        });
    }
    get terminalParams() {
        const requestParams = getRequestParams();
        const params = Object.assign(Object.assign({}, requestParams), this.pagingParams);
        return params;
    }
    get subAccountParams() {
        let accountIdNumbers = this.terminals.map((terminal) => terminal.account_id);
        let uniqueAccountIds = [...new Set(accountIdNumbers)];
        let accountIdString = uniqueAccountIds.join(',');
        return { sub_account_id: accountIdString };
    }
    get entityId() {
        return this.terminals.map((terminal) => terminal.id);
    }
    get showEmptyState() {
        return !this.loading && !this.errorMessage && this.terminalsTable.rowData.length < 1;
    }
    get showErrorState() {
        return !this.loading && !!this.errorMessage;
    }
    get showRowData() {
        return !this.showEmptyState && !this.showErrorState && !this.loading;
    }
    render() {
        return (h(TableWrapper, { key: 'feaf37a5952ecbb6bd9c1bc1a0aa1239e809fa14' }, h(TableComponent, { key: '4f10f57069b2303cf4367cd43af6683385d48c93' }, h(TableHead, { key: '527818c903322fca90a9b30b48f46fdf9c51a4a1' }, h(TableHeadRow, { key: '4b5c0cf99b36e0224193efd5b013e6ca5f468133' }, this.terminalsTable.columnData.map((column) => column))), h(TableBody, { key: '94f3573897fecf7c520ceb51bfd6cf68706f14c2' }, h(TableLoadingState, { key: '709fe7ac553832df64034f9b9acbe1a64004c99b', columnSpan: this.terminalsTable.columnKeys.length, isLoading: this.loading }), h(TableEmptyState, { key: '25c050dfaaf86a3792e276c27938a066b014f619', isEmpty: this.showEmptyState, columnSpan: this.terminalsTable.columnKeys.length }), h(TableErrorState, { key: '6aeca4b673c92d095e7d871ca5acb10f66df9b7e', columnSpan: this.terminalsTable.columnKeys.length, errorMessage: this.errorMessage }), this.showRowData && this.terminalsTable.rowData.map((data, index) => (h(TableRow, { "data-test-id": "table-row", "data-row-entity-id": this.entityId[index], onClick: this.rowClickHandler }, data)))), this.paging && (h(TableFoot, { key: '89554de16c42f1016b7aa6173b6b89bf3e3159db' }, h(TableFootRow, { key: '97b8cbebcb3bc6152c2f1329a00ed6a44f785cbe' }, h(TableFootCell, { key: 'd299aa13feba175e46fd8ed9883232e9145ece52', colSpan: this.terminalsTable.columnData.length }, h("pagination-menu", { key: '01a3d9ee669d1865c167d2fc9a2ca9166b984a6c', paging: Object.assign(Object.assign({}, this.paging), { handleClickPrevious: this.handleClickPrevious, handleClickNext: this.handleClickNext }) }))))))));
    }
    static get watchers() { return {
        "pagingParams": ["updateOnPropChange"],
        "getTerminals": ["updateOnPropChange"],
        "getSubAccounts": ["updateOnPropChange"],
        "columns": ["updateOnPropChange"]
    }; }
}, [0, "terminals-list-core", {
        "getTerminals": [16, "get-terminals"],
        "getSubAccounts": [16, "get-sub-accounts"],
        "columns": [1],
        "terminals": [32],
        "terminalsTable": [32],
        "subAccounts": [32],
        "loading": [32],
        "errorMessage": [32],
        "paging": [32],
        "pagingParams": [32]
    }, undefined, {
        "pagingParams": ["updateOnPropChange"],
        "getTerminals": ["updateOnPropChange"],
        "getSubAccounts": ["updateOnPropChange"],
        "columns": ["updateOnPropChange"]
    }]);
function defineCustomElement() {
    if (typeof customElements === "undefined") {
        return;
    }
    const components = ["terminals-list-core", "pagination-menu"];
    components.forEach(tagName => { switch (tagName) {
        case "terminals-list-core":
            if (!customElements.get(tagName)) {
                customElements.define(tagName, TerminalsListCore);
            }
            break;
        case "pagination-menu":
            if (!customElements.get(tagName)) {
                defineCustomElement$1();
            }
            break;
    } });
}
defineCustomElement();

export { TerminalsListCore as T, defaultColumnsKeys as a, defineCustomElement as d };
